<?php
/*
Plugin Name: Google Analytics Link Builder
Plugin URI: http://www.businessxpand.com
Description: Build search engine friendly tracking links, trackable with through your Google Analytics account
Author: Business Xpand
Version: 0.9
Author URI: http://www.businessxpand.com
*/
// error_reporting(E_ALL);
// ini_set('display_errors', '1');

/**
 * Google Analytics Link Builder plugin file
 *
 * LICENSE: The GNU General Public License (GPL)
 *
 * @copyright  2009 Business Xpand
 * @license    GPL v2.0
 * @version    0.9
 * @link       http://www.businessxpand.com
 * @since      File available since Release 0.9
*/
class BxGaLinkBuilder
{
    /**
     * Constructor
     *
     * @since 0.9
     *
     * @param null
     * @return void
     */
    function BxGaLinkBuilder()
    {
        register_activation_hook(__FILE__, array(&$this, 'activate'));
        add_action('init', array(&$this, 'init'));
        add_filter('generate_rewrite_rules', array(&$this, 'generateRewriteRules'));
        add_filter('query_vars', array(&$this, 'queryVars'));
        if (is_admin()) {
            add_action('admin_menu',array(&$this,'admin'));
        } else {
            add_action('wp_head', array(&$this, 'wpHead'), 1);
        }
    }

    /**
     * Plugin activation
     *
     * @since 0.9
     *
     * @param null
     * @return void
     */
    function activate()
    {
        global $wpdb;

        $databaseVersion = 0.9;

        if (($bxGaLinksOptions = get_option('bxgalinks')) === false) {
            $bxGaLinksOptions['dbversion'] = $databaseVersion;
            add_option('bxgalinks', $bxGaLinksOptions);
            $this->createDefaults();
        }

        if (($wpdb->get_var("SHOW TABLES LIKE '".$wpdb->prefix."bxgalinks'") != $wpdb->prefix."bxgalinks") || ($bxGaLinksOptions['dbversion'] != $databaseVersion)) {
            $sql = "CREATE TABLE `".$wpdb->prefix."bxgalinks` (
                    `id` BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
                    `rewrite_term` VARCHAR(255) NOT NULL,
                    `redirect_url` VARCHAR(255) NOT NULL,
                    `utm_source` VARCHAR(255) NOT NULL,
                    `utm_medium` VARCHAR(255) NOT NULL,
                    `utm_term` VARCHAR(255) NOT NULL,
                    `utm_content` VARCHAR(255) NOT NULL,
                    `utm_campaign` VARCHAR(255) NOT NULL,
                    PRIMARY KEY (`id`),
                    UNIQUE KEY `rewrite_term` (`rewrite_term`));";
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            dbDelta($sql);
            if (!isset($bxGaLinksOptions['dbversion']) || ($bxGaLinksOptions['dbversion'] != $databaseVersion)) {
                $bxGaLinksOptions['dbversion'] = $databaseVersion;
                update_option('bxgalinks', $bxGaLinksOptions);
            }
        }
        if (($wpdb->get_var("SHOW TABLES LIKE '".$wpdb->prefix."bxgastats'") != $wpdb->prefix."bxgastats") || ($bxGaLinksOptions['dbversion'] != $databaseVersion)) {
            $sql = "CREATE TABLE `".$wpdb->prefix."bxgastats` (
        						`id` BIGINT(20) NOT NULL AUTO_INCREMENT,
						        `datetime` DATETIME NOT NULL,
						        `link_id` BIGINT(20) NOT NULL,
						        `http_host` VARCHAR(255) NOT NULL,
                    `http_user_agent` VARCHAR(255) NOT NULL,
                    `http_referer` VARCHAR(255) NOT NULL,
                    `remote_addr` VARCHAR(255) NOT NULL,
                    PRIMARY KEY (`id`));";
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            dbDelta($sql);
            if (!isset($bxGaLinksOptions['dbversion']) || ($bxGaLinksOptions['dbversion'] != $databaseVersion)) {
                $bxGaLinksOptions['dbversion'] = $databaseVersion;
                update_option('bxgalinks', $bxGaLinksOptions);
            }
        }
    }

    /**
     * Populate defaults
     *
     * @since 0.9
     *
     * @param null
     * @return void
     */
    function createDefaults()
    {
        $defaults = array('prefix' => 'bxgalink',
                          'exclude_ips' => $_SERVER['REMOTE_ADDR']);

        $bxGaLinksOptions = get_option('bxgalinks');
        foreach ($defaults as $key => $value) {
            $bxGaLinksOptions[$key] = $value;
        }
        update_option('bxgalinks', $bxGaLinksOptions);
    }

    /**
     * Initilise rules
     *
     * @since 0.9
     *
     * @param null
     * @return void
     */
    function init()
    {
        global $wp_rewrite;
   	    $wp_rewrite->flush_rules();
    }

    /**
     * Add query variable
     *
     * @since 0.9
     *
     * @param array $publicQueryVars
     * @return array
     */
    function queryVars($publicQueryVars)
    {
        $bxGaLinksOptions = get_option('bxgalinks');
        $publicQueryVars[] = $bxGaLinksOptions['prefix'];
        return $publicQueryVars;
    }

    /**
     * Create permalinks
     *
     * @since 0.9
     *
     * @param object $wp_rewrite
     * @return void
     */
    function generateRewriteRules($wp_rewrite)
    {
        $bxGaLinksOptions = get_option('bxgalinks');
        $rules = array($bxGaLinksOptions['prefix'] . '/(.+)/?$' => 'index.php?' . $bxGaLinksOptions['prefix'] . '=$matches[1]',
                       $bxGaLinksOptions['prefix'] . '/?$' => 'index.php');
        $wp_rewrite->rules = $rules + $wp_rewrite->rules;
    }

    /**
     * Add menu pages
     *
     * @since 0.9
     *
     * @param null
     * @return void
     */
    function admin()
    {
        add_menu_page('Google Analytics Links Builder &rsaquo; Setup', 'Links Builder', 'level_7', __FILE__, array(&$this,'adminMenu'));
        add_submenu_page(__FILE__, 'Setup Links', 'Setup Links', 'level_7', 'bxgalinks-links', array(&$this,'adminLinks'));
    }

    /**
     * Main admin menu
     *
     * @since 0.9
     *
     * @param null
     * @return stdout
     */
    function adminMenu()
    {
        $bxGaLinksOptions = get_option('bxgalinks');
        if (isset($_REQUEST['action']) && ($_REQUEST['action']=='save')) {
            check_admin_referer('bxgalinks-nonce');
            if (isset($_REQUEST['bxgalinksform'])) {
                if (count($_REQUEST['bxgalinks'])>0) {
                    foreach ($_REQUEST['bxgalinks'] as $key => $value) {
                        $bxGaLinksOptions[$key] = $value;
                    }
                    update_option('bxgalinks', $bxGaLinksOptions);
                }
            }
        }
?><div class='wrap'>
    <h2>Google Analytics Link Builder</h2>
    <h3>Instructions</h3>
    <ul>
        <li>To get the most out of this plugin, permalinks should be switched on.</li>
        <li>You need to ensure that your Google Analytics javascript code is on the page that you are redirecting to.</li>
    </ul>
    <div>
        <form method="post" action="">
            <?php wp_nonce_field('bxgalinks-nonce'); ?>
            <input type="hidden" name="action" value="save"/>
            <input type="hidden" name="bxgalinksform" value="true"/>
            <table class="form-table">
                <tbody>
                    <tr valign="top">
                        <th scope="row"><label for="prefix">Link prefix</label></th>
                        <td><input type="text" id="prefix" name="bxgalinks[prefix]" value="<?php echo (isset($bxGaLinksOptions['prefix']) ? $bxGaLinksOptions['prefix'] : ''); ?>"/></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <label for="exclude_ips">Exclude IPs</label><br/>
                            <small>Place the IP addresses to exclude from the tracking on separate lines.</small>
                        </th>
                        <td><textarea id="exclude_ips" name="bxgalinks[exclude_ips]" rows="10"><?php echo (isset($bxGaLinksOptions['exclude_ips']) ? $bxGaLinksOptions['exclude_ips'] : ''); ?></textarea></td>
                    </tr>
                </tbody>
            </table>
            <p class="submit"><input class="button-primary" type="submit" name="submit" value="Save Changes"/></p>
        </form>
    </div>
</div><?php
    }

    /**
     * Admin links setup menu
     *
     * @since 0.9
     *
     * @param null
     * @return stdout
     */
    function adminLinks()
    {
        global $wpdb;
        $invalidNoRewriteTerm = false;
        $invalidNoRedirectUrl = false;
        $invalidRewriteTerm = false;
        $invalidRedirectUrl = false;
        $invalidUtm = false;
        if (isset($_REQUEST['action']) && ($_REQUEST['action']=='editlink')) {
            if (isset($_REQUEST['id']) && is_numeric($_REQUEST['id'])) {
                $this->editLinkForm($_REQUEST['id']);
            } else {
                $this->editLinkForm();
            }
        } else {
            if (isset($_REQUEST['action']) && ($_REQUEST['action']=='updatelink')) {
                check_admin_referer('bxgalinks-nonce');
                if (isset($_REQUEST['bxgalinksform'])) {
                    if (count($_REQUEST['bxgalinks'])>0) {
                        if (trim($_REQUEST['bxgalinks']['rewrite_term']) == '') $invalidNoRewriteTerm = $_REQUEST['bxgalinks']['id'];
                        if (trim($_REQUEST['bxgalinks']['redirect_url']) == '') $invalidNoRedirectUrl = $_REQUEST['bxgalinks']['id'];
                        if (trim($_REQUEST['bxgalinks']['utm_source']) != '') {
                            if ((trim($_REQUEST['bxgalinks']['utm_medium']) == '') || (trim($_REQUEST['bxgalinks']['utm_campaign']) == '')) $invalidUtm = $_REQUEST['bxgalinks']['id'];
                        }
                        if (trim($_REQUEST['bxgalinks']['utm_medium']) != '') {
                            if ((trim($_REQUEST['bxgalinks']['utm_source']) == '') || (trim($_REQUEST['bxgalinks']['utm_campaign']) == '')) $invalidUtm = $_REQUEST['bxgalinks']['id'];
                        }
                        if (trim($_REQUEST['bxgalinks']['utm_campaign']) != '') {
                            if ((trim($_REQUEST['bxgalinks']['utm_source']) == '') || (trim($_REQUEST['bxgalinks']['utm_medium']) == '')) $invalidUtm = $_REQUEST['bxgalinks']['id'];
                        }
                        if (!$invalidNoRewriteTerm && !$invalidNoRedirectUrl) {
                            if (!$this->checkUrl($_REQUEST['bxgalinks']['redirect_url'])) {
                                $invalidRedirectUrl = $_REQUEST['bxgalinks']['id'];
                            } else {
                                $sqlSet = '';
                                foreach ($_REQUEST['bxgalinks'] as $key => $value) {
                                    if ($key == 'rewrite_term') {
                                        $value = str_replace(array('?', '&', '=', '/', ':', '@'), '-', $value);
                                        $value = urlencode($value);
                                    }
                                    if ($key != 'id') {
                                        if (empty($sqlSet))
                                            $sqlSet .= "`$key` = '$value'";
                                        else
                                            $sqlSet .= ", `$key` = '$value'";
                                    }
                                }
                                $sql = "SELECT `id` FROM `" . $wpdb->prefix . "bxgalinks` WHERE `rewrite_term` = '{$_REQUEST['bxgalinks']['rewrite_term']}' AND `id` != '{$_REQUEST['bxgalinks']['id']}';";
                                $result = $wpdb->get_row($sql);
                                if ((count($result) > 0) && ($result->id != $_REQUEST['bxgalinks']['id'])) {
                                    $invalidRewriteTerm = $_REQUEST['bxgalinks']['id'];
                                } else {
                                    if (isset($_REQUEST['bxgalinks']['id']) && (!empty($sqlSet))) {
                                        $sql = "UPDATE `" . $wpdb->prefix . "bxgalinks` SET $sqlSet WHERE `id` = '{$_REQUEST['bxgalinks']['id']}';";
                                    } else {
                                        $sql = "INSERT INTO `" . $wpdb->prefix . "bxgalinks` SET $sqlSet;";
                                    }
                                    $wpdb->query($sql);
                                }
                            }
                        }
                    }
                }
            } else if (isset($_REQUEST['action']) && ($_REQUEST['action']=='deletelink')) {
                if (isset($_REQUEST['id']) && is_numeric($_REQUEST['id'])) {
                    $sql = "DELETE FROM `" . $wpdb->prefix . "bxgalinks` WHERE `id` = '{$_REQUEST['id']}';";
                    $wpdb->query($sql);
                }
            } else if (isset($_REQUEST['action']) && ($_REQUEST['action']=='delete-selected')) {
                if (isset($_REQUEST['doaction_active']) && isset($_REQUEST['checked'])) {
                    if (count($_REQUEST['checked'])>0) {
                        foreach ($_REQUEST['checked'] as $id) {
                            $wpdb->query("DELETE FROM `" . $wpdb->prefix . "bxgalinks` WHERE `id` = '$id';");
                        }
                    }
                }
            }
            if ($invalidNoRewriteTerm) {
                $this->editLinkForm($invalidNoRewriteTerm, $_REQUEST['bxgalinks'], 'Link suffix required.');
            } else if ($invalidNoRedirectUrl) {
                $this->editLinkForm($invalidNoRedirectUrl, $_REQUEST['bxgalinks'], 'Redirect to field is required.');
            } else if ($invalidRedirectUrl) {
                $this->editLinkForm($invalidRedirectUrl, $_REQUEST['bxgalinks'], 'The web site you are attempting to redirect to does not exist.');
            } else if ($invalidRewriteTerm) {
                $this->editLinkForm($invalidRewriteTerm, $_REQUEST['bxgalinks'], 'Link suffix already exists, please choose another word.');
            } else if ($invalidUtm) {
                $this->editLinkForm($invalidUtm, $_REQUEST['bxgalinks'], 'Google source, medium and name must be filled in, or left blank.');
            } else {
                $sql = "SELECT `id`, `rewrite_term`, `redirect_url` FROM `" . $wpdb->prefix . "bxgalinks`;";
                $results = $wpdb->get_results($sql);
                $bxGaLinksOptions = get_option('bxgalinks');
?><div class='wrap'>
    <h2>Setup Links</h2>
    <div class="tablenav">
        <a href="admin.php?page=bxgalinks-links&amp;action=editlink">Add new link</a>
    </div>
    <div>
        <form method="post" action="">
            <?php wp_nonce_field('bxgalinks-nonce'); ?>
            <input type="hidden" name="action" value="save"/>
            <input type="hidden" name="bxgalinksform" value="true"/>
            <?php if (count($results) > 0) { ?>
            <div class="tablenav">
                <div class="alignleft actions">
                    <select name="action">
                        <option selected="selected" value="">Bulk Actions</option>
                        <option value="delete-selected">Delete</option>
                    </select>
                    <input class="button-secondary action" type="submit" value="Apply" name="doaction_active"/>
                </div>
            </div>
            <div class="clear"></div>
            <table id="active-links-table" class="widefat" cellspacing="0">
                <thead>
                    <tr>
                        <th class="check-column" scope="col">
                            <input type="checkbox"/>
                        </th>
                        <th scope="col">Link</th>
                        <th scope="col">Redirects to</th>
                        <th scope="col">Clicks</th>
                        <th class="action-links" scope="col">Actions</th>
                    </tr>
                </thead>
                <tbody class="bxgalinks-links">
                    <?php foreach ($results as $result) { ?>
                    <tr>
                        <th class="check-column" scope="row">
                            <input type="checkbox" value="<?php echo $result->id; ?>" name="checked[]"/>
                        </th>
                        <td class="name">
                            <?php
                            if (get_option('permalink_structure')!='') {
                                $rewriteUrl = get_option('siteurl') . '/' . $bxGaLinksOptions['prefix'] . '/' . $result->rewrite_term;
                            } else {
                                $rewriteUrl = get_option('siteurl') . '/?' . $bxGaLinksOptions['prefix'] . '=' . $result->rewrite_term;
                            }
                            ?>
                            <?php /*<a href="admin.php?page=bxgalinks-links&amp;action=editlink&amp;id=<?php echo $result->id; ?>" title="Edit <?php echo $result->rewrite_term; ?>"><?php echo $rewriteUrl; ?></a><br/>*/ ?>
                            <input type="text" value="<?php echo $rewriteUrl; ?>" style="border:0px none;margin:0px;padding:0px;font-size:10px;width:100%;" onclick="selectAll(this)" readonly="readonly"/>
                        </td>
                        <td class="name">
                            <a href="<?php echo $result->redirect_url; ?>" target="_blank" title="Visit <?php echo $result->redirect_url; ?>"><?php echo $result->redirect_url; ?></a>
                        </td>
                        <td class="name">
                            <?php echo $this->clicks($result->id); ?>
                        </td>
                        <td class="togl action-links">
                            <a href="admin.php?page=bxgalinks-links&amp;action=editlink&amp;id=<?php echo $result->id; ?>" title="Edit <?php echo $result->rewrite_term; ?>">Edit</a>&nbsp;&nbsp;&nbsp;
                            <a href="javascript:confirmDelete('<?php echo $result->rewrite_term; ?>', 'admin.php?page=bxgalinks-links&amp;action=deletelink&amp;id=<?php echo $result->id; ?>')" title="Delete <?php echo $result->rewrite_term; ?>">Delete</a>
                        </td>
                    </tr>
                    <?php } ?>
                </tbody>
                <tfoot>
                    <tr>
                        <th class="check-column" scope="col">
                            <input type="checkbox"/>
                        </th>
                        <th scope="col">Link</th>
                        <th scope="col">Redirects to</th>
                        <th scope="col">Clicks</th>
                        <th class="action-links" scope="col">Actions</th>
                    </tr>
                </tfoot>
            </table>
            <script type="text/javascript">
            /* <![CDATA[ */
                function confirmDelete(term, url) {
                    if (confirm("Are you sure you want to delete '" + term + "'?")) {
                        document.location = url;
                    }
                }
                function selectAll(field) {
                    var textVal = eval(field);
                    textVal.focus();
                    textVal.select();
                    if (!document.all) return; // IE only
                    r = textVal.createTextRange();
                    r.execCommand("copy");
                }
            /* ]]> */
            </script>
            <?php } else { ?>
            <p>There are currently no links set.</p>
            <?php } ?>
        </form>
    </div>
</div><?php
            }
        }
    }

    /**
     * Edit links form
     *
     * @since 0.9
     *
     * @param null
     * @return stdout
     */
    function editLinkForm($id = '', $request = '', $message = '')
    {
        global $wpdb;
        $googleMoreInfo = 'http://www.google.com/support/googleanalytics/bin/answer.py?answer=55579&hl=en';
        $rewriteTerm = '';
        $redirectUrl = '';
        $utmSource = '';
        $utmMedium = '';
        $utmTerm = '';
        $utmContent = '';
        $utmCampaign = '';
        if (!empty($id)) {
            $sql = "SELECT * FROM `" . $wpdb->prefix . "bxgalinks` WHERE `id` = '$id';";
            $result = $wpdb->get_row($sql);
            if (count($result)>0) {
                $rewriteTerm = $result->rewrite_term;
                $redirectUrl = $result->redirect_url;
                $utmSource = $result->utm_source;
                $utmMedium = $result->utm_medium;
                $utmTerm = $result->utm_term;
                $utmContent = $result->utm_content;
                $utmCampaign = $result->utm_campaign;
            }
        }
        if (!empty($request)) {
            $rewriteTerm = $request['rewrite_term'];
            $redirectUrl = $request['redirect_url'];
            $utmSource = $request['utm_source'];
            $utmMedium = $request['utm_medium'];
            $utmTerm = $request['utm_term'];
            $utmContent = $request['utm_content'];
            $utmCampaign = $request['utm_campaign'];
        }
?><div class='wrap'>
    <h2><?php echo (!empty($id) ? 'Update Link' : 'Add Link'); ?></h2>
	  <?php if (!empty($message)) { ?><div id="message" class="updated fade"><p><strong><?php echo $message; ?></strong></p></div><?php } ?>
    <div class="tablenav">
        <a href="admin.php?page=bxgalinks-links">Cancel <?php echo (!empty($id) ? 'update link' : 'add link'); ?></a>
    </div>
    <div>
        <?php wp_nonce_field('bxgalinks-nonce'); ?>
        <input type="hidden" name="action" value="addlink"/>
        <input type="hidden" name="bxgalinksform" value="true"/>
        <div class="clear"></div>
        <form method="post" action="">
            <?php wp_nonce_field('bxgalinks-nonce'); ?>
            <input type="hidden" name="action" value="updatelink"/>
            <?php if (!empty($id)) { ?><input type="hidden" name="bxgalinks[id]" value="<?php echo $id; ?>"/><?php } ?>
            <input type="hidden" name="bxgalinksform" value="true"/>
            <table class="form-table">
                <tbody>
                    <tr valign="top">
                        <th scope="row" colspan="2">
                            <p><strong>* Required fields.</strong></p>
                        </th>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <label for="rewrite_term">Link suffix *</label><br/>
                            <small>This is the term comes at the end of your campaign link url. This will appear after the 'prefix' value you set on the main setup page.</small>
                        </th>
                        <td><input type="text" id="rewrite_term" name="bxgalinks[rewrite_term]" value="<?php echo $rewriteTerm; ?>"/></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <label for="redirect_url">Redirect to *</label><br/>
                            <small>This is the web page your want your campaign link to redirect to.</small>
                        </th>
                        <td><input type="text" id="redirect_url" name="bxgalinks[redirect_url]" value="<?php echo $redirectUrl; ?>"/></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row" colspan="2">
                            <h3>Google Analytics</h3>
                            <p>The following is only necessary if you wish to track your link through your Google Analytics account. The source, medium and name are the only required fields.</p>
                            <p><strong>Important:</strong> You must have your Google Analytics javascript code on the page you are redirecting to.</p>
                        </th>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <label for="utm_source">Google source</label><br/>
                            <small><a href="<?php echo $googleMoreInfo; ?>" target="_blank">More info</a></small>
                        </th>
                        <td><input type="text" id="utm_source" name="bxgalinks[utm_source]" value="<?php echo $utmSource; ?>"/></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <label for="utm_medium">Google medium</label><br/>
                            <small><a href="<?php echo $googleMoreInfo; ?>" target="_blank">More info</a></small>
                        </th>
                        <td><input type="text" id="utm_medium" name="bxgalinks[utm_medium]" value="<?php echo $utmMedium; ?>"/></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <label for="utm_term">Google term</label><br/>
                            <small><a href="<?php echo $googleMoreInfo; ?>" target="_blank">More info</a></small>
                        </th>
                        <td><input type="text" id="utm_term" name="bxgalinks[utm_term]" value="<?php echo $utmTerm; ?>"/></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <label for="utm_content">Google content</label><br/>
                            <small><a href="<?php echo $googleMoreInfo; ?>" target="_blank">More info</a></small>
                        </th>
                        <td><input type="text" id="utm_content" name="bxgalinks[utm_content]" value="<?php echo $utmContent; ?>"/></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <label for="utm_campaign">Google name</label><br/>
                            <small><a href="<?php echo $googleMoreInfo; ?>" target="_blank">More info</a></small>
                        </th>
                        <td><input type="text" id="utm_campaign" name="bxgalinks[utm_campaign]" value="<?php echo $utmCampaign; ?>"/></td>
                    </tr>
                </tbody>
            </table>
            <p class="submit"><input class="button-primary" type="submit" name="submit" value="<?php echo (!empty($id) ? 'Update link' : 'Add link'); ?>"/></p>
        </form>
    </div>
</div><?php
    }

    /**
     * Add redirecting to header
     *
     * @since 0.9
     *
     * @param null
     * @return void
     */
    function wpHead()
    {
        global $wpdb;
        $bxGaLinksOptions = get_option('bxgalinks');
        if ($bxGaLink = get_query_var($bxGaLinksOptions['prefix'])) {
            $sql = "SELECT * FROM `" . $wpdb->prefix . "bxgalinks` WHERE `rewrite_term` = '$bxGaLink';";
            $result = $wpdb->get_row($sql);
            if (count($result)>0) {
                $redirectUrl = $result->redirect_url;
                if (strpos($redirectUrl, '?')) $queryDivider = '&'; else $queryDivider = '?';
                if (!empty($result->utm_source) || !empty($result->utm_medium) || !empty($result->utm_term) || !empty($result->utm_content) || !empty($result->utm_campaign)) {
                    $redirectUrl .= $queryDivider;
                    $redirectUrl .= (!empty($result->utm_source) ? '?utm_source=' . $result->utm_source : '') .
                                    (!empty($result->utm_medium) ? '&utm_medium=' . $result->utm_medium : '') .
                                    (!empty($result->utm_term) ? '&utm_term=' . $result->utm_term : '') .
                                    (!empty($result->utm_content) ? '&utm_content=' . $result->utm_content : '') .
                                    (!empty($result->utm_campaign) ? '&utm_campaign=' . $result->utm_campaign : '');
                }
                $this->postStatistic($result->id);
                if ($redirectUrl = $this->checkUrl($redirectUrl)) {
                    wp_redirect($redirectUrl);
                    die();
                }
            }
        }
    }

    /**
     * Post statistic to the stats table
     *
     * @since 0.9
     *
     * @param int $linkId
     * @return void
     */
    function postStatistic($linkId = '')
    {
        global $wpdb;
        $bxGaLinksOptions = get_option('bxgalinks');
        $excludeIps = explode("\n", $bxGaLinksOptions['exclude_ips']);
        foreach ($excludeIps as $excludeIp) {
            $excludeIpsCleaned[] = trim($excludeIp);
        }
        if (!empty($linkId) && is_numeric($linkId)) {
            if (!in_array($_SERVER['REMOTE_ADDR'], $excludeIpsCleaned)) {
                $sql = "SELECT `id` FROM `".$wpdb->prefix."bxgastats`
                        WHERE `http_host` = '" . (isset($_SERVER['HTTP_HOST']) ? mysql_real_escape_string($_SERVER['HTTP_HOST']) : '') . "'
                        AND `http_user_agent` = '" . (isset($_SERVER['HTTP_USER_AGENT']) ? mysql_real_escape_string($_SERVER['HTTP_USER_AGENT']) : '') . "'
                        AND `http_referer` = '" . (isset($_SERVER['HTTP_REFERER']) ? mysql_real_escape_string($_SERVER['HTTP_REFERER']) : '') . "'
                        AND `remote_addr` = '" . (isset($_SERVER['REMOTE_ADDR']) ? mysql_real_escape_string($_SERVER['REMOTE_ADDR']) : '') . "';";
                $results = $wpdb->get_results($sql);
                if (count($results)==0) {
                    $sql = "INSERT INTO `".$wpdb->prefix."bxgastats` SET
                           `datetime` = NOW(),
                           `link_id` = '$linkId',
                           `http_host` = '" . (isset($_SERVER['HTTP_HOST']) ? mysql_real_escape_string($_SERVER['HTTP_HOST']) : '') . "',
                           `http_user_agent` = '" . (isset($_SERVER['HTTP_USER_AGENT']) ? mysql_real_escape_string($_SERVER['HTTP_USER_AGENT']) : '') . "',
                           `http_referer` = '" . (isset($_SERVER['HTTP_REFERER']) ? mysql_real_escape_string($_SERVER['HTTP_REFERER']) : '') . "',
                           `remote_addr` = '" . (isset($_SERVER['REMOTE_ADDR']) ? mysql_real_escape_string($_SERVER['REMOTE_ADDR']) : '') . "';";
                    $wpdb->query($sql);
                }
            }
        }
    }

    /**
     * Return number of clicks on the links
     *
     * @since 0.9
     *
     * @param int $linkId
     * @return int
     */
    function clicks($linkId = '')
    {
        global $wpdb;
        if (!empty($linkId) && is_numeric($linkId)) {
            $sql = "SELECT COUNT(`id`) as clicks FROM `".$wpdb->prefix."bxgastats` WHERE `id` = '$linkId';";
            $result = $wpdb->get_row($sql);
            $return = $result->clicks;
        } else {
            $return = 0;
        }
        return $return;
    }

    function checkUrl($url)
    {
        if (substr(trim($url),0,4)!='http') $newUrl = 'http://'.trim($url); else $newUrl = $url;
        if (strlen($newUrl)) {
            $file = @fopen($newUrl, "r");
            if ($file) $return = $newUrl; else $return = false;
        } else {
            $return = false;
        }
        if (!$return) {
            $newUrl = get_option('siteurl') . '/' . trim($url, '/');
            $file = @fopen($newUrl, "r");
            if ($file) $return = $newUrl; else $return = false;
        }
        return $return;
    }
}
$bxGaLinkBuilder = new BxGaLinkBuilder;
